//
//      Yasuharu Okamoto, 2019.10
//
functions {
    real half_normal_lpdf(real y, real sgm){
        if (y > 0.0){
            return normal_lpdf(y | 0.0, sgm);
        } else {
            return log(0.0);
        }
    }

    real Ex_Wald_lpdf(real t, real mu, real a, real g){
        real g_exp;
        real F;
        real k1;
        k1 = sqrt(mu*mu - 2.0 * g);
        g_exp = g * exp(-g * t + a * (mu - k1));
        F = normal_cdf((k1 * t - a)/sqrt(t), 0.0, 1.0) +
                exp(2.0 * a * k1) *
                normal_cdf(-(k1 * t + a)/ sqrt(t), 0.0, 1.0);
        return log(g_exp * F);
    }
}
data {
    int N;
    real RT[N];
}
parameters {
    real<lower = 0.0> k;
    real<lower = 0.0> a;
    real<lower = 0.0> g;
}
transformed parameters {
    real<lower = 0.0> mu;
    mu = sqrt(k*k + 2.0*g);
}
model {
    k ~ normal(0.0, 1000);
    a ~ half_normal(1000);
    g ~ half_normal(1000);
    for(i in 1:N){
        RT[i]  ~ Ex_Wald(mu, a, g);
    }
}
