import numpy as np
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import pystan
import pickle

#
#           データの読み込み
#
flnm = input('Data file name(*.csv) = ')
fin = open(flnm, 'r')
data = fin.readlines()
fin.close()

N = len(data) - 1
DataID = []
RTs = []
for i in range(N):
    str = data[i+1].split(',')
    DataID.append(str[0])
    RTs.append(float(str[1]))

for i in range(N):
    print(DataID[i],'  ', RTs[i])

RTs = np.array(RTs)

sm = pystan.StanModel(file = 'Ex_Wald.stan')

Data = {'N': N, 'RT': RTs}

fit = sm.sampling(data = Data, n_jobs = 1)

print(fit)

k = fit['k']
a = fit['a']
g = fit['g']
mu = fit['mu']

plt.hist(a)
plt.title(r'Posterior Distribution of $\alpha$', fontsize = 18)
plt.xlabel(r'$\alpha$', fontsize = 14)
plt.show()

plt.hist(g)
plt.title(r'Posterior Distribution of $\gamma$', fontsize = 18)
plt.xlabel(r'$\gamma$', fontsize = 14)
plt.show()

plt.hist(mu)
plt.title(r'Posterior Distribution of $\mu$', fontsize = 18)
plt.xlabel(r'$\mu$', fontsize = 14)
plt.show()

k_med = np.percentile(k, 50)
a_med = np.percentile(a, 50)
g_med = np.percentile(g, 50)
mu_med = np.percentile(mu, 50)

import math
def ex_wald( t, mu, a, g ):
    w = []
    for v in t:
        if v <= 0.0:
            w.append(0.0)
        else:
            k = (mu**2 - 2*g) ** 0.5
            g_exp = g * math.exp(-g * v + (a * (mu - k)))
            F = ss.norm.cdf((k * v - a) / (v**0.5)) +\
                    math.exp(2.0 * a * k) *\
                    ss.norm.cdf(-(k * v + a) / (v**0.5))
            w.append(g_exp * F)
    return np.array(w)

L_x = RTs.min() - (RTs.max() - RTs.min()) * 0.1
U_x = RTs.max() + (RTs.max() - RTs.min()) * 0.1
x = np.arange(L_x, U_x, 0.001)
y = ex_wald(x, mu_med, a_med, g_med)
plt.plot(x, y, linewidth = 3, label = 'ex-Wald')
plt.hist(RTs, bins = 10, density = True, label = 'RT')
plt.xlabel('Time', fontsize = 14)
plt.title(r'ex-Wald($\mu$,$\sigma$=1,$\alpha$,$\gamma$)' + '\n' +\
          (r'$\mu$ = {0:<.3f}  $\alpha$ = {1:<.3f}  $\gamma$ = {2:<.3f}'.
          format(mu_med, a_med, g_med)), fontsize = 16)
plt.legend(fontsize = 14)
plt.show()

